
/*
 *	Routines to indentify additional cpu features that are scattered in
 *	cpuid space.
 */

#include <linux/cpu.h>
#include <linux/init.h>

#include <asm/processor.h>
#include <asm/cpufeature.h>

struct cpuid_bit {
	u16 feature;
	u8 reg;
	u8 bit;
	u32 level;
};

enum cpuid_regs {
	CR_EAX = 0,
	CR_ECX,
	CR_EDX,
	CR_EBX
};

#ifdef CONFIG_SPEC_CTRL
void get_scattered_cpuid_extra(struct cpuinfo_x86 *c)
{
	u32 max_level;
	u32 regs[4];
	const struct cpuid_bit *cb;

	/*
	 * CPUID query table for features scattered across
	 * levels > 0x6
	 */
	static const struct cpuid_bit cpuid_bits[] = {
		{ X86_FEATURE_SMEP,	    CR_EBX,  7, 0x00000007 },
		{ X86_FEATURE_SPEC_CTRL,    CR_EDX, 26, 0x00000007 },
		{ X86_FEATURE_INTEL_STIBP,  CR_EDX, 27, 0x00000007 },
		{ X86_FEATURE_FLUSH_L1D,  CR_EDX, 28, 0x00000007 },
		{ X86_FEATURE_ARCH_CAPABILITIES,  CR_EDX, 29, 0x00000007 },
		{ X86_FEATURE_SPEC_CTRL_SSBD,  CR_EDX, 31, 0x00000007 },
		{ X86_FEATURE_AMD_SSB_NO,    CR_EBX, 26, 0x80000008 },
		{ X86_FEATURE_AMD_IBPB, CR_EDX, 12, 0x80000008 },
		{ X86_FEATURE_AMD_IBRS, CR_EBX, 14, 0x80000008 },
		{ X86_FEATURE_AMD_STIBP, CR_EBX, 15, 0x80000008 },
		{ X86_FEATURE_VIRT_SSBD, CR_EBX, 25, 0x80000008 },
		{ X86_FEATURE_AMD_SSB_NO, CR_EBX, 26, 0x80000008 },
		{ 0, 0, 0, 0 }
	};

	for (cb = cpuid_bits; cb->feature; cb++) {

		/* Verify that the level is valid */
		max_level = cpuid_eax(cb->level & 0xffff0000);
		if ((max_level < cb->level) || (max_level > (cb->level | 0xffff)))
			continue;

		cpuid_count(cb->level, 0, &regs[CR_EAX], &regs[CR_EBX],
			    &regs[CR_ECX], &regs[CR_EDX]);

		if (regs[cb->reg] & (1 << cb->bit))
			rh_cpu_set_cap(cb->feature);
	}
}
#else /* CONFIG_SPEC_CTRL */
 #define get_scattered_cpuid_extra(c) { }
#endif

void __cpuinit init_scattered_cpuid_features(struct cpuinfo_x86 *c)
{
	u32 max_level;
	u32 regs[4];
	const struct cpuid_bit *cb;

	static const struct cpuid_bit cpuid_bits[] = {
		{ X86_FEATURE_IDA, CR_EAX, 1, 0x00000006 },
		{ X86_FEATURE_ARAT, CR_EAX, 2, 0x00000006 },
		{ X86_FEATURE_CPB, CR_EDX, 9, 0x80000007 },
		{ 0, 0, 0, 0 }
	};

	for (cb = cpuid_bits; cb->feature; cb++) {

		/* Verify that the level is valid */
		max_level = cpuid_eax(cb->level & 0xffff0000);
		if (max_level < cb->level ||
		    max_level > (cb->level | 0xffff))
			continue;

		cpuid(cb->level, &regs[CR_EAX], &regs[CR_EBX],
			&regs[CR_ECX], &regs[CR_EDX]);

		if (regs[cb->reg] & (1 << cb->bit))
			set_bit(cb->feature, c->x86_capability);
	}

	/*
	 * Extended features CPUID query (EAX=7, ECX=0) has to be
	 * performed separately in rhel5, in order to keep the kABI
	 */
	get_scattered_cpuid_extra(c);
}
