/*
 * Routines to deal with aditional CPU bugs related to SPECULATIVE BRANCHING
 * Copyright Red Hat, Inc. 2018
 */

#include <linux/init.h>
#include <linux/sysdev.h>
#include <linux/module.h>
#include <asm/processor.h>
#include <asm/cpufeature.h>
#include <asm-generic/nospec-branch.h>
#include <asm-generic/spec_ctrl.h>
#include <asm-i386/cpu_device_id.h>
#include <asm-i386/intel-family.h>
#include <asm/e820.h>
#include <linux/mod_devicetable.h>
#include <linux/cpu.h>
#include <linux/prctl.h>

/*
 * CPU bug word
 */
unsigned long __cpu_bugs __read_mostly = 0;
EXPORT_SYMBOL_GPL(__cpu_bugs);

/*
 * CPU speculative execution control mitigation flags
 */
unsigned long __cpu_smelt_flags __read_mostly = 0;

static const struct x86_cpu_id cpu_no_speculation[] = {
	{ X86_VENDOR_INTEL,	6, INTEL_FAM6_ATOM_CEDARVIEW,	X86_FEATURE_ANY },
	{ X86_VENDOR_INTEL,	6, INTEL_FAM6_ATOM_CLOVERVIEW,	X86_FEATURE_ANY },
	{ X86_VENDOR_INTEL,	6, INTEL_FAM6_ATOM_LINCROFT,	X86_FEATURE_ANY },
	{ X86_VENDOR_INTEL,	6, INTEL_FAM6_ATOM_PENWELL,	X86_FEATURE_ANY },
	{ X86_VENDOR_INTEL,	6, INTEL_FAM6_ATOM_PINEVIEW,	X86_FEATURE_ANY },
	{ X86_VENDOR_CENTAUR,	5 },
	{ X86_VENDOR_INTEL,	5 },
	{ X86_VENDOR_NSC,	5 },
	{ X86_VENDOR_ANY,	4 },
	{}
};

static const struct x86_cpu_id cpu_no_meltdown[] = {
	{ X86_VENDOR_AMD },
	{}
};

/* Only list CPUs which speculate but are non susceptible to SSB */
static const struct x86_cpu_id cpu_no_spec_store_bypass[] = {
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_SILVERMONT1	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_AIRMONT		},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_SILVERMONT2	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_MERRIFIELD	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_CORE_YONAH		},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_XEON_PHI_KNL		},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_XEON_PHI_KNM		},
	{ X86_VENDOR_AMD,	0x12,					},
	{ X86_VENDOR_AMD,	0x11,					},
	{ X86_VENDOR_AMD,	0x10,					},
	{ X86_VENDOR_AMD,	0xf,					},
	{}
};

static const struct x86_cpu_id cpu_no_l1tf[] = {
	/* in addition to cpu_no_speculation */
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_SILVERMONT1	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_SILVERMONT2	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_AIRMONT		},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_MERRIFIELD	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_MOOREFIELD	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_GOLDMONT	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_DENVERTON	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_ATOM_GEMINI_LAKE	},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_XEON_PHI_KNL		},
	{ X86_VENDOR_INTEL,	6,	INTEL_FAM6_XEON_PHI_KNM		},
	{}
};

static void __init cpu_set_bug_bits(struct cpuinfo_x86 *c)
{
	u64 ia32_cap = 0;

	if (x86_match_cpu(cpu_no_speculation))
		return;

	setup_force_cpu_bug(X86_BUG_SPECTRE_V1);
	setup_force_cpu_bug(X86_BUG_SPECTRE_V2);

	if (rh_cpu_test_cap(X86_FEATURE_ARCH_CAPABILITIES))
		rdmsrl(MSR_IA32_ARCH_CAPABILITIES, ia32_cap);

	if (rh_cpu_test_cap(X86_FEATURE_ARCH_CAPABILITIES))
		rdmsrl(MSR_IA32_ARCH_CAPABILITIES, ia32_cap);

	if (!x86_match_cpu(cpu_no_spec_store_bypass) &&
	   !(ia32_cap & ARCH_CAP_SSB_NO) &&
	   !rh_cpu_test_cap(X86_FEATURE_AMD_SSB_NO))
		setup_force_cpu_bug(X86_BUG_SPEC_STORE_BYPASS);

	if (x86_match_cpu(cpu_no_meltdown))
		return;

	/* Rogue Data Cache Load? No! */
	if (ia32_cap & ARCH_CAP_RDCL_NO)
		return;

	setup_force_cpu_bug(X86_BUG_CPU_MELTDOWN);
	if (x86_match_cpu(cpu_no_l1tf))
		return;
	setup_force_cpu_bug(X86_BUG_L1TF);
}

void __cpuinit init_cpu_bugs(struct cpuinfo_x86 *c)
{
	cpu_set_bug_bits(c);
}

void x86_amd_ssbd_enable(void)
{
#ifdef CONFIG_SPEC_CTRL
	u64 msrval = x86_amd_ls_cfg_base | x86_amd_ls_cfg_ssbd_mask;

	if (rh_cpu_test_cap(X86_FEATURE_VIRT_SSBD))
		wrmsrl(MSR_AMD64_VIRT_SPEC_CTRL, SPEC_CTRL_SSBD);
	else if (rh_cpu_test_cap(X86_FEATURE_LS_CFG_SSBD))
		wrmsrl(MSR_AMD64_LS_CFG, msrval);
#endif
}

/* The kernel command line selection */
enum spectre_v2_mitigation_cmd {
	SPECTRE_V2_CMD_NONE,
	SPECTRE_V2_CMD_AUTO,
	SPECTRE_V2_CMD_FORCE,
	SPECTRE_V2_CMD_RETPOLINE,
	SPECTRE_V2_CMD_RETPOLINE_IBRS_USER,
	SPECTRE_V2_CMD_IBRS,
	SPECTRE_V2_CMD_IBRS_ALWAYS,
};

static const char *spectre_v2_strings[] = {
	[SPECTRE_V2_NONE]                       = "Vulnerable",
	[SPECTRE_V2_RETPOLINE_MINIMAL]          = "Vulnerable: Minimal ASM retpoline",
	[SPECTRE_V2_RETPOLINE_NO_IBPB]          = "Vulnerable: Retpoline without IBPB",
	[SPECTRE_V2_RETPOLINE_UNSAFE_MODULE]	= "Vulnerable: Retpoline with unsafe module(s)",
	[SPECTRE_V2_RETPOLINE]			= "Mitigation: Full retpoline",
	[SPECTRE_V2_RETPOLINE_IBRS_USER]        = "Mitigation: Full retpoline and IBRS (user space)",
	[SPECTRE_V2_IBRS]			= "Mitigation: IBRS (kernel)",
	[SPECTRE_V2_IBRS_ALWAYS]		= "Mitigation: IBRS (kernel and user space)",
	[SPECTRE_V2_IBP_DISABLED]		= "Mitigation: IBP disabled",
};

#undef pr_fmt
#define pr_fmt(fmt)     "Spectre V2: " fmt

static enum spectre_v2_mitigation spectre_v2_enabled __read_mostly = SPECTRE_V2_NONE;
static enum spectre_v2_mitigation spectre_v2_retpoline __read_mostly = SPECTRE_V2_NONE;
static enum spectre_v2_mitigation_cmd spectre_v2_cmd __read_mostly = SPECTRE_V2_CMD_AUTO;

static inline bool match_option(const char *arg, int arglen, const char *opt)
{
	int len = strlen(opt);

	return len == arglen && !strncmp(arg, opt, len);
}

static void __init spec2_print_if_insecure(const char *reason)
{
	if (boot_cpu_has_bug(X86_BUG_SPECTRE_V2))
		pr_info("%s\n", reason);
}

static void __init spec2_print_if_secure(const char *reason)
{
	if (!boot_cpu_has_bug(X86_BUG_SPECTRE_V2))
		pr_info("%s\n", reason);
}

static int __init set_nospectre_v2(char *arg)
{
	spectre_v2_cmd = SPECTRE_V2_CMD_NONE;
	return 0;
}
early_param("nospectre_v2", set_nospectre_v2);

static int __init set_spectre_v2(char *arg)
{
	if (!arg)
		return 0;
	if (!strcmp(arg, "off")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_NONE;
	} else if (!strcmp(arg, "on")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_FORCE;
	} else if (!strcmp(arg, "retpoline")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_RETPOLINE;
	} else if (!strcmp(arg, "retpoline,ibrs_user")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_RETPOLINE_IBRS_USER;
	} else if (!strcmp(arg, "ibrs")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_IBRS;
	} else if (!strcmp(arg, "ibrs_always")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_IBRS_ALWAYS;
	} else if (!strcmp(arg, "auto")) {
		spectre_v2_cmd = SPECTRE_V2_CMD_AUTO;
	}
	return 0;
}
early_param("spectre_v2", set_spectre_v2);

void spectre_v2_report_unsafe_module(struct module *mod)
{
	if (retp_compiler())
		printk_once(KERN_WARNING
			    "WARNING: module '%s' built without retpoline-enabled "
			    "compiler, may affect Spectre v2 mitigation\n",
			    mod->name);

	if (spectre_v2_retpoline == SPECTRE_V2_RETPOLINE)
		spectre_v2_retpoline = SPECTRE_V2_RETPOLINE_UNSAFE_MODULE;

	if (spectre_v2_enabled == SPECTRE_V2_RETPOLINE)
		spectre_v2_enabled = SPECTRE_V2_RETPOLINE_UNSAFE_MODULE;
}

static enum spectre_v2_mitigation_cmd __init spectre_v2_parse_cmdline(void)
{
	switch (spectre_v2_cmd) {
	case SPECTRE_V2_CMD_NONE:
		spec2_print_if_insecure("disabled on command line.");
		break;

	case SPECTRE_V2_CMD_AUTO:
		break;

	case SPECTRE_V2_CMD_IBRS:
		spec2_print_if_insecure("ibrs selected on command line.");
		break;

	case SPECTRE_V2_CMD_IBRS_ALWAYS:
		spec2_print_if_insecure("ibrs_always selected on command line.");
		break;

	case SPECTRE_V2_CMD_FORCE:
		spec2_print_if_secure("force enabled on command line.");
		break;

	case SPECTRE_V2_CMD_RETPOLINE:
		spec2_print_if_insecure("retpoline selected on command line.");
		break;

	case SPECTRE_V2_CMD_RETPOLINE_IBRS_USER:
		spec2_print_if_insecure("retpoline (kernel) and ibrs (user) selected on command line.");
		break;
	}
	return spectre_v2_cmd;
}

static inline void setup_minimal_retpl(void)
{
	spectre_v2_enabled = SPECTRE_V2_RETPOLINE_MINIMAL;
	rh_cpu_clear_cap(X86_FEATURE_RETPOLINE);
}

void __spectre_v2_select_mitigation(void)
{
	enum spectre_v2_mitigation_cmd cmd = spectre_v2_cmd;
	const bool full_retpoline = is_enabled_retp() && retp_compiler();
	spectre_v2_enabled = SPECTRE_V2_NONE;

	/*
	 * If the CPU is not affected and the command line mode is NONE or AUTO
	 * then nothing to do.
	 */
	if (!boot_cpu_has_bug(X86_BUG_SPECTRE_V2) &&
	    (cmd == SPECTRE_V2_CMD_NONE || cmd == SPECTRE_V2_CMD_AUTO))
		return;

	switch (cmd) {
	case SPECTRE_V2_CMD_NONE:
		/*
		 * We cannot totally shut off retpolines in RHEL-5, if we built
		 * with CONFIG_RETPOLINE=y, in which case we still have minimal
		 * ASM retpolines, even when the cmdline bears boot time option
		 * nospectre_v2 or spectre_v2=off
		 */
		setup_minimal_retpl();
		return;

	case SPECTRE_V2_CMD_FORCE:
		/* FALLTRHU */
	case SPECTRE_V2_CMD_AUTO:
		goto auto_mode;

	case SPECTRE_V2_CMD_RETPOLINE_IBRS_USER:
		/* FALLTRHU */
	case SPECTRE_V2_CMD_RETPOLINE:
		if (is_enabled_retp())
			goto retpoline;
		break;

	case SPECTRE_V2_CMD_IBRS:
		if (spec_ctrl_force_enable_ibrs())
			return;
		break;

	case SPECTRE_V2_CMD_IBRS_ALWAYS:
		if (spec_ctrl_enable_ibrs_always() ||
		    spec_ctrl_force_enable_ibp_disabled())
			return;
		break;
	}
	pr_err("kernel not compiled with retpoline; no mitigation available!");
	return;

auto_mode:
	if (spec_ctrl_cond_enable_ibrs(full_retpoline))
		return;

	spec_ctrl_cond_enable_ibp_disabled();

retpoline:
	if (spectre_v2_enabled != SPECTRE_V2_NONE ||
	    spectre_v2_retpoline == SPECTRE_V2_RETPOLINE_UNSAFE_MODULE)
		goto retpoline_ibrs_user;

	spectre_v2_enabled = retp_compiler()
			   ? SPECTRE_V2_RETPOLINE
			   : SPECTRE_V2_RETPOLINE_MINIMAL;
	rh_cpu_set_cap(X86_FEATURE_RETPOLINE);
	spectre_v2_retpoline = spectre_v2_enabled;

	/*
	 * Enable RETPOLINE_IBRS_USER mode, if necessary.
	 */
retpoline_ibrs_user:
	if (cmd == SPECTRE_V2_CMD_RETPOLINE_IBRS_USER)
		spec_ctrl_enable_retpoline_ibrs_user();
}

enum spectre_v2_mitigation spectre_v2_get_mitigation(void)
{
	return spectre_v2_enabled;
}

void spectre_v2_set_mitigation(enum spectre_v2_mitigation mode)
{
	spectre_v2_enabled = mode;
}

void spectre_v2_print_mitigation(void)
{
	pr_info("%s\n", spectre_v2_strings[spectre_v2_enabled]);
}

bool spectre_v2_has_full_retpoline(void)
{
	return spectre_v2_retpoline == SPECTRE_V2_RETPOLINE;
}

/*
 * Reset to the original retpoline setting when IBRS is dyamically disabled.
 */
void spectre_v2_retpoline_reset(void)
{
	spectre_v2_enabled = spectre_v2_retpoline;
}

void __init spectre_v2_select_mitigation(void)
{
	spectre_v2_parse_cmdline();
	__spectre_v2_select_mitigation();
	spectre_v2_print_mitigation();
}
#undef pr_fmt
#define pr_fmt(fmt)    "Speculative Store Bypass: " fmt

enum ssb_mitigation ssb_mode __read_mostly = SPEC_STORE_BYPASS_NONE;
EXPORT_SYMBOL_GPL(ssb_mode);

/* The kernel command line selection */
enum ssb_mitigation_cmd {
	SPEC_STORE_BYPASS_CMD_NONE,
	SPEC_STORE_BYPASS_CMD_AUTO,
	SPEC_STORE_BYPASS_CMD_ON,
	SPEC_STORE_BYPASS_CMD_PRCTL,
	SPEC_STORE_BYPASS_CMD_SECCOMP,
};

const char *ssb_strings[] = {
	[SPEC_STORE_BYPASS_NONE]	= "Vulnerable",
	[SPEC_STORE_BYPASS_DISABLE]	= "Mitigation: Speculative Store Bypass disabled",
	[SPEC_STORE_BYPASS_PRCTL]	= "Mitigation: Speculative Store Bypass disabled via prctl",
	[SPEC_STORE_BYPASS_SECCOMP]	= "Mitigation: Speculative Store Bypass disabled via prctl and seccomp",
};

static enum ssb_mitigation_cmd  ssb_cmd = SPEC_STORE_BYPASS_CMD_AUTO;

static int __init set_no_ssbd_disable(char *arg)
{
	ssb_cmd = SPEC_STORE_BYPASS_CMD_NONE;
	return 0;
}
early_param("nospec_store_bypass_disable", set_no_ssbd_disable);

static int __init set_ssbd_disable(char *arg)
{
	if (!arg)
		return 0;

	if (!strcmp(arg, "off")) {
		ssb_cmd = SPEC_STORE_BYPASS_CMD_NONE;
	} else if (!strcmp(arg, "on")) {
		ssb_cmd = SPEC_STORE_BYPASS_CMD_ON;
	} else if (!strcmp(arg, "auto")) {
		ssb_cmd = SPEC_STORE_BYPASS_CMD_AUTO;
	} else if (!strcmp(arg, "prctl")) {
		ssb_cmd = SPEC_STORE_BYPASS_CMD_PRCTL;
	} else if (!strcmp(arg, "seccomp")) {
		ssb_cmd = SPEC_STORE_BYPASS_CMD_SECCOMP;
	}
	return 0;
}
early_param("spec_store_bypass_disable", set_ssbd_disable);

static enum ssb_mitigation __ssb_select_mitigation(void)
{
	enum ssb_mitigation mode = SPEC_STORE_BYPASS_NONE;
	enum ssb_mitigation_cmd cmd = ssb_cmd;

	if (!rh_cpu_test_cap(X86_FEATURE_SSBD))
		return mode;

	if (!boot_cpu_has_bug(X86_BUG_SPEC_STORE_BYPASS) &&
	    (cmd == SPEC_STORE_BYPASS_CMD_NONE ||
	     cmd == SPEC_STORE_BYPASS_CMD_AUTO))
		return mode;

	switch (cmd) {
	case SPEC_STORE_BYPASS_CMD_AUTO:
	case SPEC_STORE_BYPASS_CMD_SECCOMP:
		/*
		 * Choose prctl+seccomp as the default mode if seccomp is
		 * enabled.
		 */
#if defined(CONFIG_SECCOMP)
			mode = SPEC_STORE_BYPASS_SECCOMP;
#else
			mode = SPEC_STORE_BYPASS_PRCTL;
#endif
		break;
	case SPEC_STORE_BYPASS_CMD_ON:
		mode = SPEC_STORE_BYPASS_DISABLE;
		break;
	case SPEC_STORE_BYPASS_CMD_PRCTL:
		mode = SPEC_STORE_BYPASS_PRCTL;
		break;
	case SPEC_STORE_BYPASS_CMD_NONE:
		break;
	}

	/*
	 * 32-bit SSBD support isn't implemented yet
	 */

#ifdef CONFIG_SPEC_CTRL
	/*
	 * We have three CPU feature flags that are in play here:
	 *  - X86_BUG_SPEC_STORE_BYPASS - CPU is susceptible.
	 *  - X86_FEATURE_SSBD - CPU is able to turn off speculative store bypass
	 *  - X86_FEATURE_SPEC_STORE_BYPASS_DISABLE - engage the mitigation
	 */
	if (mode == SPEC_STORE_BYPASS_DISABLE) {
		rh_cpu_set_cap(X86_FEATURE_SPEC_STORE_BYPASS_DISABLE);
		/*
		 * Intel uses the SPEC CTRL MSR Bit(2) for this, while AMD may
		 * use a completely different MSR and bit dependent on family.
		 */
		/*
		 * Always set the SSBD bit for both AMD & Intel.
		 */
		x86_spec_ctrl_base |= SPEC_CTRL_SSBD;
		if (!rh_cpu_test_cap(X86_FEATURE_MSR_SPEC_CTRL))
			x86_amd_ssbd_enable();
		else {
			x86_spec_ctrl_mask |= SPEC_CTRL_SSBD;
			wrmsrl(MSR_IA32_SPEC_CTRL, x86_spec_ctrl_base);
		}
	}
#endif
	return mode;
}

void ssb_print_mitigation()
{
	pr_info("%s\n", ssb_strings[ssb_mode]);
}

void ssb_select_mitigation(void)
{
	ssb_mode = __ssb_select_mitigation();

	if (boot_cpu_has_bug(X86_BUG_SPEC_STORE_BYPASS))
		ssb_print_mitigation();
}

#undef pr_fmt
#define pr_fmt(fmt)     "Speculation prctl: " fmt

static int ssb_prctl_set(struct task_struct *task, unsigned long ctrl)
{
#ifdef CONFIG_SPEC_CTRL
	bool update;

	if (ssb_mode != SPEC_STORE_BYPASS_PRCTL &&
	    ssb_mode != SPEC_STORE_BYPASS_SECCOMP)
		return -ENXIO;

	switch (ctrl) {
	case PR_SPEC_ENABLE:
		/* If speculation is force disabled, enable is not allowed */
		if (task_spec_ssb_force_disable(task))
			return -EPERM;
		task_clear_spec_ssb_disable(task);
		update = test_and_clear_tsk_thread_flag(task, TIF_SSBD);
		break;
	case PR_SPEC_DISABLE:
		task_set_spec_ssb_disable(task);
		update = !test_and_set_tsk_thread_flag(task, TIF_SSBD);
		break;
	case PR_SPEC_FORCE_DISABLE:
		task_set_spec_ssb_disable(task);
		task_set_spec_ssb_force_disable(task);
		update = !test_and_set_tsk_thread_flag(task, TIF_SSBD);
		break;
	default:
		return -ERANGE;
	}

	/*
	 * If being set on non-current task, delay setting the CPU
	 * mitigation until it is next scheduled.
	 */
	if (task == current && update)
		speculative_store_bypass_update_current();

	return 0;
#else
	return -ENXIO;
#endif
}

static int ssb_prctl_get(struct task_struct *task)
{
#ifdef CONFIG_SPEC_CTRL
	switch (ssb_mode) {
	case SPEC_STORE_BYPASS_DISABLE:
		return PR_SPEC_DISABLE;
	case SPEC_STORE_BYPASS_SECCOMP:
	case SPEC_STORE_BYPASS_PRCTL:
		if (task_spec_ssb_force_disable(task))
			return PR_SPEC_PRCTL | PR_SPEC_FORCE_DISABLE;
		if (task_spec_ssb_disable(task))
			return PR_SPEC_PRCTL | PR_SPEC_DISABLE;
		return PR_SPEC_PRCTL | PR_SPEC_ENABLE;
	default:
		if (boot_cpu_has_bug(X86_BUG_SPEC_STORE_BYPASS))
			return PR_SPEC_ENABLE;
		return PR_SPEC_NOT_AFFECTED;
	}
#else
	return PR_SPEC_NOT_AFFECTED;
#endif
}

int arch_prctl_spec_ctrl_set(struct task_struct *task, unsigned long which,
			     unsigned long ctrl)
{
	switch (which) {
	case PR_SPEC_STORE_BYPASS:
		return ssb_prctl_set(task, ctrl);
	default:
		return -ENODEV;
	}
}

#ifdef CONFIG_SECCOMP
void arch_seccomp_spec_mitigate(struct task_struct *task)
{
	if (ssb_mode == SPEC_STORE_BYPASS_SECCOMP)
		ssb_prctl_set(task, PR_SPEC_FORCE_DISABLE);
}
#endif


int arch_prctl_spec_ctrl_get(struct task_struct *task, unsigned long which)
{
	switch (which) {
	case PR_SPEC_STORE_BYPASS:
		return ssb_prctl_get(task);
	default:
		return -ENODEV;
	}
}


#ifdef CONFIG_SPEC_CTRL
#undef pr_fmt
#define pr_fmt(fmt)     "L1TF: " fmt
void l1tf_select_mitigation(void)
{
	u64 half_pa;

	if (!boot_cpu_has_bug(X86_BUG_L1TF))
		return;

	/*
	 * This is extremely unlikely to happen because almost all
	 * systems have far more MAX_PA/2 than RAM can be fit into
	 * DIMM slots.
	 */
	half_pa = (u64)l1tf_pfn_limit() << PAGE_SHIFT;
	if (e820_any_mapped(half_pa, ULLONG_MAX - half_pa, E820_RAM)) {
		pr_warn("System has more than MAX_PA/2 memory. L1TF mitigation not effective.\n");
		return;
	}

	rh_cpu_set_cap(X86_FEATURE_L1TF_PTEINV);
}
#undef pr_fmt
#endif

#ifdef CONFIG_SYSFS
ssize_t cpu_show_meltdown(struct sysdev_class *class, char *buf)
{
	if (!boot_cpu_has_bug(X86_BUG_CPU_MELTDOWN))
		return sprintf(buf, "Not affected\n");
	if (rh_cpu_test_cap(X86_FEATURE_PTI))
		return sprintf(buf, "Mitigation: PTI\n");
	return sprintf(buf, "Vulnerable\n");
}

ssize_t cpu_show_spectre_v1(struct sysdev_class *class, char *buf)
{
	if (!boot_cpu_has_bug(X86_BUG_SPECTRE_V1))
		return sprintf(buf, "Not affected\n");
	/*
	 * Load fences have been added in various places within the RHEL5 kernel
	 * to mitigate this vulnerability. See CVE-2017-5753 RHBZ 1527434.
	 */
	return sprintf(buf, "Mitigation: Load fences\n");
}

ssize_t cpu_show_spectre_v2(struct sysdev_class *class, char *buf)
{
	if (!boot_cpu_has_bug(X86_BUG_SPECTRE_V2))
		return sprintf(buf, "Not affected\n");
	return sprintf(buf, "%s%s\n", spectre_v2_strings[spectre_v2_enabled],
		       ibpb_enabled() ?  ", IBPB" : "");
}

ssize_t cpu_show_spec_store_bypass(struct sysdev_class *class, char *buf)
{
	if (!boot_cpu_has_bug(X86_BUG_SPEC_STORE_BYPASS))
		return sprintf(buf, "Not affected\n");
	return sprintf(buf, "%s\n", ssb_strings[ssb_mode]);
}

ssize_t cpu_show_l1tf(struct sys_device *dev, char *buf)
{
	if (!rh_cpu_test_cap(X86_FEATURE_L1TF_PTEINV))
		if (!boot_cpu_has_bug(X86_BUG_L1TF))
			return sprintf(buf, "Not affected\n");
		else
			return sprintf(buf, "Vulnerable\n");

	return sprintf(buf, "Mitigation: Page Table Inversion\n");
}

static SYSDEV_CLASS_ATTR(meltdown, 0400, cpu_show_meltdown, NULL);
static SYSDEV_CLASS_ATTR(spectre_v1, 0400, cpu_show_spectre_v1, NULL);
static SYSDEV_CLASS_ATTR(spectre_v2, 0400, cpu_show_spectre_v2, NULL);
static SYSDEV_CLASS_ATTR(spec_store_bypass, 0400, cpu_show_spec_store_bypass, NULL);
static SYSDEV_CLASS_ATTR(l1tf, 0400, cpu_show_l1tf, NULL);

static struct attribute *cpu_root_vulnerabilities_attrs[] = {
	&attr_meltdown.attr,
	&attr_spectre_v1.attr,
	&attr_spectre_v2.attr,
	&attr_spec_store_bypass.attr,
	&attr_l1tf.attr,
	NULL
};

static const struct attribute_group cpu_root_vulnerabilities_group = {
	.name  = "vulnerabilities",
	.attrs = cpu_root_vulnerabilities_attrs,
};

#define pr_fmt(fmt) fmt
void __init cpu_register_vulnerabilities(struct sysdev_class *dev)
{
#ifdef CONFIG_GENERIC_CPU_VULNERABILITIES
	if (sysfs_create_group(&dev->kset.kobj, &cpu_root_vulnerabilities_group))
		pr_err("Unable to register CPU vulnerabilities\n");
#endif /* CONFIG_GENERIC_CPU_VULNERABILITIES */
}

#undef pr_fmt

#ifdef CONFIG_SPEC_CTRL
__u32 cpu_caps_cleared[RHNCAPINTS];
__u32 cpu_caps_set[RHNCAPINTS];
#endif

#endif /* CONFIG_SYSFS */
